﻿// <copyright file="frmCheckIn.cs" company="Microsoft Corporation"> Copyright © Microsoft Corporation. All Rights Reserved. This code released under the terms of the Microsoft Public License (MS-PL, http://opensource.org/licenses/ms-pl.html.) This is sample code only, do not use in production environments</copyright>

// Visual Studio ALM Ranger Sample Source Code
//
// Project Name: ALMRangersCheckInForOtherUser
// File Name: frmCheckInForOtherUser.cs
//
// Description: Sample code to demonstrate some of the APIs of the Team Foundation
//              Server Object Model. This sample demsontrates connecting to a server
//              using the local registered servers, using the version control service
//              to get a list of projects, creating workspaces, adding files to
//              workspace, and checking them in as another user. There is no or 
//              minimal error handling in the sample.
//
// Author(s): Brian Blacklman
//
// This is sample code for demonstration purposes only and therefore the intent is to
// demonstrate key features and may not contain coding best practices. 
// Put this code through your own quality standards and testing process before using 
// this code in production systems.
namespace CheckInForOtherUser
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net;
    using System.Windows.Forms;
    using Microsoft.TeamFoundation.Client;
    using Microsoft.TeamFoundation.VersionControl.Client;

    public partial class frmCheckIn : Form
    {
        private VersionControlServer vcServer;

        public frmCheckIn()
        {
            this.InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Populate the form with collections registered in the registry
            // for the current user and get all registered collections on this machine
            List<RegisteredProjectCollection> regProjectCollections = 
                new List<RegisteredProjectCollection>(RegisteredTfsConnections.GetProjectCollections());

            // Enumerate the list of collectons and add to combo box
            foreach (var projectCollection in regProjectCollections)
            {
                this.cbRegColl.Items.Add(projectCollection.Uri);
            }
        }

        private void cbRegColl_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Clear List Box before populating it
            this.lbProjects.Items.Clear();

            // Get the selected collection
            ComboBox cb = (ComboBox)sender;
            int index = cb.SelectedIndex;

            // Connect to the team project collection and the server that hosts the version-control repository. 
            using (TfsTeamProjectCollection tpc = new TfsTeamProjectCollection(new Uri(cb.Text), new UICredentialsProvider()))
            {
                // Get the version control service
                this.vcServer = tpc.GetService<VersionControlServer>();

                // Get all the team projects
                var teamProjects = new List<TeamProject>(this.vcServer.GetAllTeamProjects(false));

                // Enumerate the team projects in the collection and add to listbox
                foreach (TeamProject projectNode in teamProjects)
                {
                    this.lbProjects.Items.Add(projectNode.Name);
                }
            }
        }

        private void lbProjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Using visibility to control form workflow
            this.lblUserId.Visible = true;
            this.tbUserID.Visible = true;
            this.btnCheckIn.Visible = true;
        }

        private void btnCheckIn_Click(object sender, EventArgs e)
        {
            // This is the key code where we use key SCM features such as workspaces
            // and adding files to our local workspace and then checking them in
            // as another user
            Workspace workspace;

            if (this.tbUserID.TextLength < 1)
            {
                return;
            }

            // Create a workspace
            string workspaceName = string.Format("{0}-{1}", Environment.MachineName, @"Test");

            try
            {
                workspace = this.vcServer.GetWorkspace(workspaceName, this.vcServer.AuthorizedUser);
            }
            catch (WorkspaceNotFoundException)
            {
                workspace = this.vcServer.CreateWorkspace(workspaceName, this.vcServer.AuthorizedUser);
            }

            var serverFolder = string.Format("$/{0}", this.lbProjects.Text);
            var localFolder = Path.Combine(Path.GetTempPath(), @"VCTest");
            var workingFolder = new WorkingFolder(serverFolder, localFolder);

            // Create a workspace mapping
            workspace.CreateMapping(workingFolder);

            // Get the files from the repository
            workspace.Get();

            // Create a file
            string newFolder = Path.Combine(workspace.Folders[0].LocalItem, "For Test Purposes");
            Directory.CreateDirectory(newFolder);
            string newFilename = Path.Combine(newFolder, "Safe to Delete.txt");

            if (!workspace.HasCheckInPermission)
            {
                throw new SecurityException(string.Format("{0} does not have check-in permission for workspace: {1}", workspace.VersionControlServer.AuthorizedUser, workspace.DisplayName));
            }

            // create the new file
            using (var streamWriter = new StreamWriter(newFilename))
            {
                streamWriter.WriteLine("Revision 1");
            }

            // Now pend the add of our new folder and file
            workspace.PendAdd(Path.GetDirectoryName(newFilename), true);
            var pendingAdds = new List<PendingChange>(workspace.GetPendingChanges());

            // Checkin the items we added
            workspace.CheckIn(pendingAdds.ToArray(), this.tbUserID.Text, "Initial check-in", null, null, null);
        }
    }
}
